; ------------------------------------------------------------------------------
;
;					winPenPack Mozilla builder
;
; ------------------------------------------------------------------------------
;
; AutoIt Version:			3.2.12.1
; Platform:					Windows
; Script Name:				winPenPack Mozilla builder
; Script Function:			Installazione applicazioni mozilla
; Script Version:			1.1.0.0
; Author:					Gabriele Tittonel <tittoproject@gmail.com>
; Modified:					Energy (multilingua, licenza, gestione errori, tray)
; Home Page:				http://www.winpenpack.com
; License:					GNU General Public License
; Contributors:				winPenPack Team
;
; ------------------------------------------------------------------------------

; Tray icon exit
#NoTrayIcon
Opt("MustDeclareVars", 1)
Opt("TrayAutoPause", 0)
Opt("TrayOnEventMode",1)
Opt("TrayMenuMode",1)
TrayCreateItem("Exit")
TrayItemSetOnEvent(-1,"ExitScript")
TraySetState()

; Script Info
Global $ScriptName = 'winPenPack Mozilla builder'
TraySetToolTip($ScriptName)

; App Info
Global $AppName = EnvGet('X_MOZILLA')
Global $AppFile = StringLower($AppName) & '.exe'
Global $AppNameFull = 'X-' & $AppName
Global $LicensePath = @ScriptDir & '/LICENSE'

; X-Launcher
Global $LauncherExe = EnvGet('X_LAUNCHER')
Global $LauncherIni = EnvGet('X_CONFIG')

; Directories
Global $RootDir = EnvGet('X_ROOT')
Global $OutDir = EnvGet('X_OUTDIR')

; Check if process is already running
If ProcessExists($AppFile) Then
	MsgBox(0, $ScriptName, $AppName & " is already running." & @LF & _
			"Please close it and try again.")
	Exit 2
EndIf

; Language
Global $LangDefault = EnvGet('LANG')
If $LangDefault = '' Then $LangDefault='it'
Global $LangOther = ''
Global $SetupLang = _FindLang(@ScriptDir, $LangDefault)
If @error Then _Error_Msg("Setup directory not found!")
Global $Lang = $SetupLang[0][1]
Global $MsgInfoText, $MsgInfoTitle, $MsgInfoAccept, $MsgInfoLang, $MsgInfoLicense, $MsgInfoCancel, $MsgInfoRun
_MsgInfo($Lang)
Dim $LangSel = False
If $SetupLang[0][0] > 1 Then $LangSel = True
_SelectLang($LangSel)

; Setup
Global $SetupDir = @ScriptDir & '\setup_' & $Lang
Global $SetupName = _FindSetup($SetupDir, $AppName)
If @error Then _Error_Msg("Setup file not found!")
Global $SetupFile = $SetupDir & '\' & $SetupName
If Not FileExists($SetupFile) Or StringInStr(FileGetAttrib($SetupFile), 'D') Then _Error_Msg("Setup file not found!")

; Extract setup
If $OutDir = "" Then _Error_Msg("Instal directory not found!")
DirRemove($OutDir, 1)
DirCreate($OutDir)
Local $iError = RunWait('7za x "' & $SetupFile & '" -o"' & $OutDir & '"', $SetupDir, @SW_HIDE)
If @error Then _Error_Msg("7za not found!")
If $iError > 0 Then _Error_Msg("Installation failed!")
If FileChangeDir($OutDir) = 0 Then _Error_Msg("WorkingDir not found!")

FileDelete('*')
_DirMove('nonlocalized', '.', 1)
_DirMove('localized', '.', 1)
_DirMove('optional', '.', 1)
; _DirMove(@ScriptDir & '\dictionaries_it_en', 'dictionaries', 1)
; DirRemove('uninstall')

; Config
Global $ConfigDefault = $SetupDir & '\' & $AppNameFull & '.ini'
FileCopy($ConfigDefault, $LauncherIni, 1)

Local $XDrive = @ScriptDir & '\..\..\..\XDrive'
For $i = 1 To $SetupLang[0][0]
;~ 	Local $ConfigOther = $SetupLang[$i][0] & '\' & $AppNameFull & '.ini'
;~ 	If FileExists ($XDrive) Then
;~ 		FileCopy($ConfigOther, $XDrive & '\' & $SetupLang[$i][1] & '\' & $AppNameFull & '.ini', 1)
;~ 	EndIf
	If $SetupLang[$i][2] Then
;~ 		If $Lang <> $SetupLang[$i][1] Then
;~ 			RunWait('7za e "' & $SetupLang[$i][2] & '" -o"' & $OutDir & '\chrome" "localized\chrome\*" -aos', @ScriptDir, @SW_HIDE)
;~ 			RunWait('7za e "' & $SetupLang[$i][2] & '" -o"' & $OutDir & '\dictionaries" "localized\dictionaries\*" -aos', @ScriptDir, @SW_HIDE)
;~ 			RunWait('7za e "' & $SetupLang[$i][2] & '" -o"' & $OutDir & '\searchplugins" "localized\searchplugins\*" -aos', @ScriptDir, @SW_HIDE)
;~ 		EndIf
		FileDelete($SetupLang[$i][2])
	EndIf
	; DirCopy(@ScriptDir & '\chrome_' & $SetupLang[$i][1], 'chrome', 1)
	; DirCopy(@ScriptDir & '\searchplugins_' & $SetupLang[$i][1], 'searchplugins', 1)
	; DirCopy(@ScriptDir & '\profile_' & $SetupLang[$i][1], 'defaults\profile', 1)
Next

; Start X-Mozilla
Dim $Parameters = _CmdLineParam()
Run($LauncherExe & $Parameters)
GUIDelete()
GUIDelete()

#Region Funzioni

Func ExitScript()
	GUIDelete()
	GUIDelete()
	MsgBox(262192,$AppNameFull, "Installation interrupted by user!")
    Exit 1
EndFunc

Func _Error_Msg($sText, $fContinue=False)
	GUIDelete()
	GUIDelete()
	if Not $fContinue Then
		MsgBox(262160,$AppNameFull & ": Error",$sText)
		Exit 3
	EndIf
	MsgBox(262192,$AppNameFull & ": Error",$sText, 4)
	Return False
EndFunc   ;==>_Error_Msg

Func _CmdLineParam($aParam = $CmdLine)
	Local $sParam = ''
	If $aParam[0] >= 1 Then
		For $i = 1 To $aParam[0]
			Local $sTags = StringLeft($aParam[$i], 1)
			Local $sQuote = '"'
			If $sTags = '-' Or $sTags = '/' Then $sQuote = ''
			$sParam &= ' ' & $sQuote & $aParam[$i] & $sQuote
		Next
	EndIf
	Return $sParam
EndFunc   ;==>_CmdLineParam

Func _FindLang($path, $sLang)
	Local $dirList = _FileList($path, 'setup*', 2)
	If @error Or $dirList[0] = 0 Then Return SetError(1,0,0)
	Local $aLang[UBound($dirList)][3]
	$aLang[0][0] = $dirList[0]
	For $i = 1 To $dirList[0]
		$aLang[$i][0] = $path & '\' & $dirList[$i]
		$aLang[$i][1] = StringRight($dirList[$i], 2)
		Local $SetupName = _FindSetup($aLang[$i][0], $AppName)
		If @error Then
			$aLang[$i][2] = False
		Else
			$aLang[$i][2] = $aLang[$i][0] & '\' & $SetupName
			$LangOther &= $aLang[$i][1] & '|'
			If $aLang[0][1] = '' Then $aLang[0][1] = $aLang[$i][1]
			If $aLang[$i][1] = $sLang Then $aLang[0][1] = $sLang
		EndIf
	Next
	$LangOther = StringTrimRight($LangOther, 1)
	Return $aLang
EndFunc   ;==>_FindLang

Func _FindSetup($path, $name)
	Local $search = FileFindFirstFile($path & '\' & $name & '*.exe')
	If $search = -1 Then Return SetError(1,0,1)
	Local $find = FileFindNextFile($search)
	If @error Then Return SetError(1,0,0)
	FileClose($search)
	Return $find
EndFunc   ;==>_FindSetup

Func _DirMove($source, $dest, $flag = 0)
	If StringInStr('/\', StringRight($dest, 1)) Then
		If FileExists($dest) Then
			Return DirMove($source, $dest, $flag)
		Else
			$dest &= _DirName($source)
		EndIf
	EndIf
	
	Local $destCreate = DirCreate($dest)
	If Not $destCreate Then Return
	FileMove($source & '\*', $dest, $flag)
	
	Local $dirList = _FileList($source, '*', 2)
	If Not @error Then
		For $i = 1 To $dirList[0]
			If FileExists($dest & '\' & $dirList[$i]) Then
				_DirMove($source & '\' & $dirList[$i], $dest & '\' & $dirList[$i], $flag)
			Else
				DirMove($source & '\' & $dirList[$i], $dest & '\' & $dirList[$i], $flag)
			EndIf
			DirRemove($source & '\' & $dirList[$i])
		Next
	EndIf
	DirRemove($source)
EndFunc   ;==>_DirMove

Func _DirName($path)
	$path = _StringStripTrailingSlash($path)
	Return StringTrimLeft($path, StringInStr($path, '\', 2, -1))
EndFunc   ;==>_DirName

Func _StringStripTrailingSlash($string)
	If StringRight(StringReplace($string, '/', '\'), 1) = '\' Then
		Return _StringStripTrailingSlash(StringTrimRight($string, 1))
	EndIf
	Return $string
EndFunc   ;==>_StringStripTrailingSlash

Func _FileList($path, $filter = '*', $flag = 0)
	If Not FileExists($path) Then Return SetError(1, 0, 0)
	If Not StringInStr(FileGetAttrib($path), 'D') Then Return SetError(2, 0, 0)
	Local $splitFilter = StringSplit($filter, ' '), $list[1]
	For $i = 1 To $splitFilter[0]
		Local $handle = FileFindFirstFile($path & '\' & $splitFilter[$i])
		While 1
			Local $file = FileFindNextFile($handle)
			If @error Then ExitLoop
			Local $isDir = _FileIsDir($path & '\' & $file)
			If $flag = 1 And $isDir = True Then ContinueLoop
			If $flag = 2 And $isDir = False Then ContinueLoop
			ReDim $list[UBound($list) + 1]
			$list[0] += 1
			$list[$list[0]] = $file
		WEnd
		FileClose($handle)
	Next
	If UBound($list) = 1 Then Return SetError(3, 0, 0)
	Return $list
EndFunc   ;==>_FileList

Func _FileIsDir($path)
	If StringInStr(FileGetAttrib($path), 'D', 2) Then Return True
	Return False
EndFunc   ;==>_FileIsDir

;~ Func _FileToArray($filepath)
;~ 	Local $read = _FileToVar($filepath)
;~ 	If Not $read Then Return
;~ 	Return StringSplit(StringStripCR($read), @LF)
;~ EndFunc   ;==>_FileToArray

;~ Func _FileToVar($file)
;~ 	Local $handle = FileOpen($file, 0)
;~ 	If $handle = -1 Then Return
;~ 	Local $read = FileRead($handle)
;~ 	FileClose($handle)
;~ 	Return $read
;~ EndFunc   ;==>_FileToVar

Func _MsgInfo($lang)
	Local $SetupDir = @ScriptDir & '\setup_' & $Lang
	Opt("ExpandVarStrings", 1)
	Switch $lang
		Case 'it'
			$MsgInfoTitle = '  ' & $AppNameFull & ': primo avvio'
			$MsgInfoText = "<b>$AppNameFull$ sta per essere avviato per la prima volta.</b><br><br>"
			$MsgInfoText &= "$AppNameFull$  una versione portatile di Mozilla $AppName$ (marchio registrato della Mozilla Foundation) creata da winPenPack.com utilizzando il file binario non modificato scaricato da Mozilla.org, "
			$MsgInfoText &= 'in accordo con la <a href="http://www.mozilla.org/foundation/trademarks/policy.html" target="_blank">"Mozilla Trademark Policy"</a>.<br>'
			$MsgInfoText &= "$AppNameFull$ contiene Mozilla $AppName$ nella sua forma originale di setup all'interno della cartella $SetupDir$.<br>"
			$MsgInfoText &= "Al primo avvio, cio alla prima esecuzione di $AppNameFull$, in automatico e in modo 'silente' verr eseguito il setup, che estrae e posiziona gli eseguibili nella cartella \Bin, inizialmente vuota, e predispone il file .ini corretto. "
			$MsgInfoText &= "Al termine dell'operazione, il setup viene rimosso dalla cartella \Lib."
			$MsgInfoAccept = 'Accetto'
			$MsgInfoCancel = 'Annulla'
			$MsgInfoLang = 'Seleziona la lingua da usare: '
			$MsgInfoLicense = '  Mozilla ' & $AppName & ': Licenza'
			$MsgInfoRun = "<center><img src='@ScriptDir@/load.gif'><br><br><span style='font-size:10px;font-family:verdana'>.. creazione versione portatile di Mozilla $AppName$ in corso ..<br><br>... attendere prego ...</span></center>"
		Case else
			$MsgInfoTitle = '  ' & $AppNameFull & ': first run'
			$MsgInfoText = "<b>$AppNameFull$ is about to be started for the first time.</b><br><br>"
			$MsgInfoText &= "$AppNameFull$ is a portable version of Mozilla $AppName$ (registered trademark of the Mozilla Foundation) created by winPenPack.com using the unchanged official binary downloaded from Mozilla.org, "
			$MsgInfoText &= 'in agreement with the <a href="http://www.mozilla.org/foundation/trademarks/policy.html" target="_blank">"Mozilla Trademark Policy"</a>.<br>'
			$MsgInfoText &= "$AppNameFull$ contains Mozilla $AppName$ in its unmodified setup form into the $SetupDir$ directory.<br>"
			$MsgInfoText &= "At first run, automatically and in 'silent' mode, will be executed the setup, extracting and moving all program files into the initially empty \Bin folder, and preparing the correct .ini file. "
			$MsgInfoText &= "At the end of installation, the setup file will be removed from \Lib folder."
			$MsgInfoAccept = 'Accept'
			$MsgInfoCancel = 'Cancel'
			$MsgInfoLang = 'Select language to use: '
			$MsgInfoLicense = '  Mozilla ' & $AppName & ': License'
			$MsgInfoRun = "<center><img src='@ScriptDir@/load.gif'><br><br><span style='font-size:10px;font-family:verdana'>.. creating portable version of Mozilla $AppName$ in progress ..<br><br>... please wait ...</span></center>"
		EndSwitch
		Opt("ExpandVarStrings", 0)
EndFunc

Func _SelectLang($LangSel)
	Local $oShell = ObjCreate("Shell.Explorer")
	Local $gui = GUICreate($MsgInfoTitle, 550, 250)
;~ 	GUISetIcon($LauncherExe)
	GUISetFont (10)
	Local $MsgInfo = GUICtrlCreateObj($oShell, 0, 0, 549, 210)
;~ 	GUICtrlSetBkColor(-1, 0xFFFFFF)
	Local $select = GUICtrlCreateLabel($MsgInfoLang, 20, 222, 200, 25)
	If Not $LangSel Then GUICtrlSetState(-1, 32)
	Local $combo = GUICtrlCreateCombo('', 200, 218, 60, 75, 0x0003)
	If Not $LangSel Then GUICtrlSetState(-1, 32)
	GUICtrlSetData($combo, $LangOther, $Lang)
	Local $button = GUICtrlCreateButton("OK", 400, 218, 90, 25)
	GUISetState(@SW_SHOW)
	With $oShell; Object tag pool
		.navigate("about:blank")
		While .busy
			Sleep(100)
		WEnd
		.document.writeln('<BODY onselectstart = "return false" oncontextmenu="return false" ondragover="return false">') 
;~ 		.document.body.title = $MsgInfoTitle
		.document.body.topmargin=5
		.document.body.leftmargin=5
		.document.body.scroll="no"
;~ 		.document.body.bgcolor = -1
		.document.body.style.fontSize = "15px"
		.document.body.innerHTML = $MsgInfoText
	EndWith
	While 1
		Switch GUIGetMsg()
			Case -3
				ExitScript()
			Case $button
				$Lang = GUICtrlRead($combo)
				ExitLoop
			Case $combo
				_MsgInfo(GUICtrlRead($combo))
				$oShell.document.body.innerHTML = $MsgInfoText
				GUICtrlSetData($select, $MsgInfoLang)
				WinSetTitle($gui, '',$MsgInfoTitle)
		EndSwitch
	WEnd
	GUICtrlDelete ($combo)
	GUICtrlDelete ($select)
	If FileExists($LicensePath) Then
		Local $sLicense = FileRead($LicensePath)
		$sLicense = StringRegExpReplace ($sLicense, '(?Ui)(http://)?(www.+)([\.,]?\s)', '<a href="http://\2" target="_blank">\2</a>\3')
		$sLicense = StringRegExpReplace ($sLicense, '\n', '<br>')
		$oShell.document.body.scroll="yes"
		$oShell.document.body.innerHTML = $sLicense
		WinSetTitle ($gui, '',$MsgInfoLicense)
		GUICtrlSetData($button, $MsgInfoAccept)
		Local $button2 = GUICtrlCreateButton($MsgInfoCancel, 250, 218, 90, 25)
		While 1
			Switch GUIGetMsg()
				Case -3
					ExitScript()
				Case $button2
					ExitScript()
				Case $button
					ExitLoop
			EndSwitch
		WEnd
	EndIf
	GUICtrlDelete ($button)
	GUICtrlDelete ($button2)
	GUICtrlDelete ($MsgInfo)
	GUIDelete($gui)
	Local $gui = GUICreate($MsgInfoTitle, 420, 150, -1, -1, BitOR(0x00800000, 0x80000000, 0x04000000), BitOR(0x00000008, 0x00000080))
	GUISetBkColor(0xFFFFFF)
	GUICtrlCreateLabel("", 0, 0, 420, 150, -1, 0x00100000)
	GUICtrlSetBkColor(-1, -2)
	GUISetState(@SW_SHOW)
	WinSetTrans($gui,"",150)
	Local $gui2 = GUICreate("IE", 420, 150, -3, -25, 0x80000000, BitOR(0x00080000, 0x00000080, 0x00000040),$gui)
	GUISetBkColor(0xFFFFFF)
	GUICtrlCreatePic(@ScriptDir & "\load.gif", 210, 18, 1, 1)
	Local $MsgInfo = GUICtrlCreateObj($oShell, 0, 0, 420, 150)
	With $oShell
		.navigate("about:blank")
		While .busy
			Sleep(100)
		WEnd
		.document.writeln('<BODY onselectstart = "return false" oncontextmenu="return false" ondragover="return false">')
		.document.body.style.border="0px"
		.document.body.topmargin=18
		.document.body.leftmargin=0
		.document.body.scroll="no"
		.document.body.style.fontSize = "16px"
		.document.body.innerHTML = $MsgInfoRun
	EndWith
	GUISetState(@SW_SHOW)
	GUISetState(@SW_DISABLE)
	$oShell = 0
	Return True
EndFunc

#EndRegion Funzioni
